'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var stream = require('stream');
var fetch = require('node-fetch');
var qs = require('qs');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fetch__default = /*#__PURE__*/_interopDefaultLegacy(fetch);
var qs__default = /*#__PURE__*/_interopDefaultLegacy(qs);

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class StackOverflowQuestionsCollatorFactory {
  constructor(options) {
    __publicField(this, "requestParams");
    __publicField(this, "baseUrl");
    __publicField(this, "apiKey");
    __publicField(this, "apiAccessToken");
    __publicField(this, "teamName");
    __publicField(this, "maxPage");
    __publicField(this, "logger");
    __publicField(this, "type", "stack-overflow");
    this.baseUrl = options.baseUrl;
    this.apiKey = options.apiKey;
    this.apiAccessToken = options.apiAccessToken;
    this.teamName = options.teamName;
    this.maxPage = options.maxPage;
    this.requestParams = options.requestParams;
    this.logger = options.logger.child({ documentType: this.type });
  }
  static fromConfig(config, options) {
    const apiKey = config.getOptionalString("stackoverflow.apiKey");
    const apiAccessToken = config.getOptionalString(
      "stackoverflow.apiAccessToken"
    );
    const teamName = config.getOptionalString("stackoverflow.teamName");
    const baseUrl = config.getOptionalString("stackoverflow.baseUrl") || "https://api.stackexchange.com/2.3";
    const maxPage = options.maxPage || 100;
    return new StackOverflowQuestionsCollatorFactory({
      baseUrl,
      maxPage,
      apiKey,
      apiAccessToken,
      teamName,
      ...options
    });
  }
  async getCollator() {
    return stream.Readable.from(this.execute());
  }
  async *execute() {
    if (!this.baseUrl) {
      this.logger.debug(
        `No stackoverflow.baseUrl configured in your app-config.yaml`
      );
    }
    if (this.apiKey && this.teamName) {
      this.logger.debug(
        "Both stackoverflow.apiKey and stackoverflow.teamName configured in your app-config.yaml, apiKey must be removed before teamName will be used"
      );
    }
    try {
      if (Object.keys(this.requestParams).indexOf("key") >= 0) {
        this.logger.warn(
          "The API Key should be passed as a separate param to bypass encoding"
        );
        delete this.requestParams.key;
      }
    } catch (e) {
      this.logger.error(`Caught ${e}`);
    }
    const params = qs__default["default"].stringify(this.requestParams, {
      arrayFormat: "comma",
      addQueryPrefix: true
    });
    const apiKeyParam = this.apiKey ? `${params ? "&" : "?"}key=${this.apiKey}` : "";
    const teamParam = this.teamName ? `${params ? "&" : "?"}team=${this.teamName}` : "";
    const requestUrl = this.apiKey ? `${this.baseUrl}/questions${params}${apiKeyParam}` : `${this.baseUrl}/questions${params}${teamParam}`;
    let hasMorePages = true;
    let page = 1;
    while (hasMorePages) {
      if (page === this.maxPage) {
        this.logger.warn(
          `Over ${this.maxPage} requests to the Stack Overflow API have been made, which may not have been intended. Either specify requestParams that limit the questions returned, or configure a higher maxPage if necessary.`
        );
        break;
      }
      const res = await fetch__default["default"](
        `${requestUrl}&page=${page}`,
        this.apiAccessToken ? {
          headers: {
            "X-API-Access-Token": this.apiAccessToken
          }
        } : void 0
      );
      const data = await res.json();
      for (const question of data.items) {
        yield {
          title: question.title,
          location: question.link,
          text: question.owner.display_name,
          tags: question.tags,
          answers: question.answer_count
        };
      }
      hasMorePages = data.has_more;
      page = page + 1;
    }
  }
}

exports.StackOverflowQuestionsCollatorFactory = StackOverflowQuestionsCollatorFactory;
//# sourceMappingURL=index.cjs.js.map
